import os

__license__ = "MIT"
__version__ = "2.1.0"
__author__ = "Zachary Zhang"
__email__ = "zlj19971222@outlook.com"
__git_url__ = "https://github.com/zlj-zz/fungit.git"
__HOME__ = os.environ["HOME"]
__FUNGITDIR__ = __HOME__ + "/.config/fungit"
